package dk.dtu.compute.se2.videoserver;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Date;

public class VideoServer {
	
	private final static String FOLDER = "tmp";
	
	public final static String VIDEO_FILE_NAME_PREFIX = "video";
    public final static String VIDEO_FILE_NAME_SUFFIX = "mp4";
    
    private final static int VIDEO_CTRL_PORT = 8802;	

			
	public static void main(String args[]) {
		String folder = FOLDER;
		if (args.length == 1) {
			folder = args[0];
		}
		
		VideoOutHandler outHandler = null;
		VideoInHandler inHandler = null;
		File file = new File(folder);
		if (file != null && file.isDirectory()) {
			outHandler = new VideoOutHandler();
			inHandler = new VideoInHandler(file, outHandler);
			outHandler.start();
			inHandler.start();
			
			System.out.println("VideoServer started in folder " + file.getAbsolutePath());
		} else {
			System.out.println("VideoServer could not be started in folder " + folder);
			System.exit(1);
		}
		
		ServerSocket controlServerSocket = null;
		Socket controlSocket = null;
		try {
			controlServerSocket = new ServerSocket(VIDEO_CTRL_PORT );
			controlSocket = controlServerSocket.accept();
			InputStream inputStream = controlSocket.getInputStream();
			
			// TODO We should actually read and interpret the incoming request
			//      For now we read (part of) the request and gobble it;
			//      Any incoming request is taken as a shutdown command.
			inputStream.read(new byte[1024]);
			
			String html = 
					"<html><body><h1>Shutting down video server!</h1></body></html>\r\n";
			
			String reply =
					"HTTP/1.1 200 OK\r\n" +
					"Server: Ekkart's simple video server\r\n" +
					"Date: " + (new Date()) + "\r\n" + 
					"Content-Type: text/html\r\n" +
					"Content-Length: " + html.length() + "\r\n" +
					"Last-Modified: " + new Date(file.lastModified()) + "\r\n" +
					"Connection: keep-alive\r\n" +
					"Accept-Ranges: bytes\r\n"+
					"\r\n" +
					html;
			
			OutputStream outputStream = controlSocket.getOutputStream();
			outputStream.write(reply.getBytes());			
		} catch(IOException e) {
			System.out.println("IOException occurred");
		}
		
		finally{
			outHandler.abort();
			inHandler.abort();
			
			if (controlServerSocket != null) {
				try {
					controlServerSocket.close();
				} catch (IOException e) {
					System.out.println("IOException occurred");
				}
			}
			if (controlSocket != null) {
				try {
					controlSocket.close();
				} catch (IOException e) {
					System.out.println("IOException occurred");
				}
			}

			
			// TODO Stopping the handlers should be done in a cleaner way.
			//      Here we just wait for some seconds for the handlers to shut
			//      down.
			try {
				Thread.sleep(5000);
			} catch (InterruptedException e) {}
			
			System.out.println("VideoServer on folder " + file.getAbsolutePath() + " terminated");
			System.exit(0);
		}
	}

}
